#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>

#include "global.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"

GtkWidget *dlgFicheros=NULL;
DISPOS *inicio=NULL;
char *fichero=NULL;

void
on_esperar_conexiones1_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *dlgespera=create_dlgEspera();
      	gtk_widget_show_now(dlgespera);
	/* Le dejo hacer sus eventos antes de esperar conexiones */
	while (gtk_events_pending ())
        	gtk_main_iteration ();
        	
	espera_conexiones();
	gtk_widget_destroy(dlgespera);	
}

void
on_cmdExaminar_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	dlgFicheros = create_dlgFicheros ();
	gtk_widget_show (dlgFicheros);
}

void
on_dlgCancelar_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(dlgFicheros);	/* Lo cierro sin mas */
}


void
on_dlgAceptar_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
        GtkWidget *txtfichero=txtFichero;
        
	fichero=gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(dlgFicheros));		/*Pillo la ruta y el fichero */
	gtk_entry_set_text(GTK_ENTRY(txtfichero),fichero);
	gtk_widget_destroy(dlgFicheros);
}

void
on_cmdDetecta_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
      GtkComboBox *cbElijeDisp;
      inquiry_info *ii = NULL; //Almacena la lista de dispositivos detectados durante el inquiry
      int max_rsp, num_rsp; //Numero de respuestas/dispositivos detectados
      int dev_id; //Identificador del adaptador Bluetooth local
      int socket; //Socket HCI;
      int len, i;
      gint a;
      char dir_mac[20];	//Es de apoyo
      char MAC_dev[20]; //Direccion MAC del dispositivo detectado
      char nombre_dev[248]; //Nombre del dispositivo detectado
      char comando[100];	/* Cadena que le paso  a system */
      DISPOS *nuevo=NULL;
      DISPOS *ultimo=NULL;
      GtkWidget *dlgBuscando;
      
      dlgBuscando=create_dlgBuscando();
      gtk_widget_show_now(dlgBuscando);
      /* Le dejo hacer sus eventos antes de buscar dispositivos */
	while (gtk_events_pending ())
        	gtk_main_iteration ();
      
      cbElijeDisp=GTK_COMBO_BOX(button);	/*Tiene que ser button en vez de user_data, por que la funcion es swapped */

      g_print("\nDetectar Dispositivos y enviar por USSP-PUSH\n");
      g_print("Si cuela... cuela\n\n");

      //Obtenemos el identificador del adaptador local Bluetooth
      dev_id = hci_get_route(NULL);
      if (dev_id < 0)
      {
            g_error("[!] Error. Dispositivo Bluetooth local no disponible.\n");
            exit(1);
      }
                     
      //Abrimos un socket local HCI
      socket = hci_open_dev(dev_id);
      if (socket < 0)
      {
            g_error("[!] Error. Fallo al intentar abrir socket HCI.\n");
            exit(1);
      }

      //Inicializamos algunas variables
      len = 10; //El tiempo de inquiry por dispositivo es de 1.28x15=19.2 secs/dispositivo
      max_rsp = 255; //Se pueden detectar a lo sumo 255 dispositivos

      //Creamos la lista de dispositivos detectados con hci_inquiry
      ii = (inquiry_info*)malloc(max_rsp * sizeof(inquiry_info));

      g_print("Detectando dispositivos...\n\n");

      //hci_inquiry lleva a cabo un descubrimiento de dispositivos Bluetooth y devuelve una lista de
      //dispositivos detectados en inquiry_info ii para ser almacenados.
      //La bandera IREQ_CACHE_FLUSH permite que la caché sea limpiada antes de buscar nuevos dispositivos.
      //En otro caso, podrian aparecer dispositivos anteriormente detectados pero ahora fuera de rango.
      num_rsp = hci_inquiry(dev_id, len, max_rsp, NULL, &ii, IREQ_CACHE_FLUSH);
      if(num_rsp < 0)
            g_error("[!] Error. Fallo al intentar hci_inquiry.\n");

	/* Si la lista ya existe la elimino */
	if (inicio!=NULL)	/* Primero elimino los elementos del combo_box y despues elimino la lista */
	{
		ultimo=inicio;
		a=0;
		while (ultimo!=NULL)	/* La lista la recorro aqui para saber cuantos elementos existen */
		{
			gtk_combo_box_remove_text (GTK_COMBO_BOX(cbElijeDisp), 0);
			ultimo=ultimo->next;
			a++;
		}
		
		elimina_lista_dispo (inicio);
		ultimo=NULL;
		inicio=NULL;
	}

 	//Para cada una de las respuestas obtenidas durante el inquiry obtenemos el nombre del dispositivo
	for(i=0; i<num_rsp; i++)
       {
		ba2str(&(ii+i)->bdaddr, MAC_dev);
            	memset(nombre_dev, 0, sizeof(nombre_dev));
            	if(hci_read_remote_name(socket, &(ii+i)->bdaddr, sizeof(nombre_dev), nombre_dev, 0) < 0)
                	strcpy(nombre_dev, "[Desconocido]");

            	g_print("Dispositivo (%d) encontrado:\n\tMAC: %s\n\tNombre: %s\n\n", i+1, MAC_dev, nombre_dev);
            	gtk_combo_box_insert_text (cbElijeDisp, i, nombre_dev);
            
            	/*Añado los datos a la lista enlazada */
            	nuevo=(DISPOS *)malloc(sizeof(DISPOS));
            	nuevo->next=NULL;
            	strcpy(nuevo->MAC_dev,MAC_dev);
            	strcpy(nuevo->nombre_dev,nombre_dev);
            	if (inicio==NULL)	/*Si es el primer elemento */
            	{
            		inicio=nuevo;
            		ultimo=nuevo;
            	}
           	 else
            	{
            		ultimo->next=nuevo;
            		ultimo=nuevo;
            	}           
	}
      
      gtk_combo_box_set_active(cbElijeDisp,0);
      free(ii);	//elimina_lista_ii (ii, num_rsp);
      //close(socket);
     gtk_widget_destroy(dlgBuscando);     
}


void
on_enviar_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	int len, i;
	inquiry_info *ii = NULL; //Almacena la lista de dispositivos detectados durante el inquiry
	char MAC_dev[20]; //Direccion MAC del dispositivo detectado
	char comando[100];	/* Cadena que le paso  a system */
	char dir_mac[20];	//Es de apoyo
	DISPOS *actual=NULL;
    GtkWidget *dlgEnvia;
	
	GtkComboBox *combo;
	gchar *nombre_dispo;
	

      
      dlgEnvia=create_dlgEnvia();
      gtk_widget_show_now(dlgEnvia);
      /* Le dejo hacer sus eventos antes de buscar dispositivos */
	while (gtk_events_pending ())
        	gtk_main_iteration ();

	
    	combo=GTK_COMBO_BOX(button);
    	nombre_dispo=gtk_combo_box_get_active_text(combo);
    	g_print("Nombre Dispositivo a enviar: %s\n",nombre_dispo);
    	
    	/*AHora buscamos el nombre en la lista*/
    	actual=inicio;
    	while (actual!=NULL)
    	{
    		if (strcmp(actual->nombre_dev,nombre_dispo)==0)	
    		{
    			strcpy(MAC_dev,actual->MAC_dev);
    			actual=NULL;
    		}
    		else
    			actual=actual->next;
    	}
    	
      	g_print("Direccion MAC: %s\n",MAC_dev);
	strcpy(dir_mac,MAC_dev);
	len=get_canal_obex (dir_mac);	/*Pillo el canal de envio */
	      
	if (len != -1)	/* No hubo error */
	{
	 	printf("Enviando...\n");
	      	sprintf(comando,"gksu rfcomm bind /dev/rfcomm0 %s %d", MAC_dev,len);
	      	system(comando);
	      	sprintf(comando,"ussp-push /dev/rfcomm0 %s %s",fichero,recorta_cadena(fichero));
	      	system(comando);
	      	system("gksu rfcomm release all");
	      	printf("saliendo...\n");
	}
	  gtk_widget_destroy(dlgEnvia);  
}

void
on_txtFichero_changed                  (GtkEditable     *editable,
                                        gpointer         user_data)
{
	GtkEntry *txtfichero=GTK_ENTRY(editable);
	
	fichero=gtk_entry_get_text(txtfichero);
}


void
on_salir_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_main_quit();
}


void
on_acerca_de_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *ventana=create_frmAcerca ();
  gtk_widget_show (ventana);
}


void
on_cbElijeDisp_changed                 (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
	GtkWidget *botonenviar=GTK_BUTTON(combobox);
	if(inicio==NULL)
    	gtk_widget_set_sensitive (botonenviar, FALSE);	/* Activo el boton */
    else
        gtk_widget_set_sensitive (botonenviar, TRUE);	/* Activo el boton */
	
}


void
on_frmPrincipal_destroy                (GtkObject       *object,
                                        gpointer         user_data)
{
  gtk_main_quit();

}

